@@ -2,32 +2,43 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
-__all__ = ['Profile', 'FirefoxProfile', 'ThunderbirdProfile']
+__all__ = ['Profile',
+           'FirefoxProfile',
+           'FirefoxMetroProfile',
+           'ThunderbirdProfile']
 
 import os
+import time
 import tempfile
+import types
+import uuid
 from addons import AddonManager
 from permissions import Permissions
-from shutil import rmtree
+from prefs import Preferences
+from shutil import copytree, rmtree
+from webapps import WebappCollection
 
 try:
-    import simplejson
+    import json
 except ImportError:
-    import json as simplejson
+    import simplejson as json
 
 class Profile(object):
     """Handles all operations regarding profile. Created new profiles, installs extensions,
     sets preferences and handles cleanup."""
 
-    def __init__(self,
-                 profile=None, # Path to the profile
-                 addons=None,  # String of one or list of addons to install
-                 addon_manifests=None,  # Manifest for addons, see http://ahal.ca/blog/2011/bulk-installing-fx-addons/
-                 preferences=None, # Dictionary or class of preferences
-                 locations=None, # locations to proxy
-                 proxy=False, # setup a proxy
-                 restore=True # If true remove all installed addons preferences when cleaning up
-                 ):
+    def __init__(self, profile=None, addons=None, addon_manifests=None, apps=None,
+                 preferences=None, locations=None, proxy=None, restore=True):
+        """
+        :param profile: Path to the profile
+        :param addons: String of one or list of addons to install
+        :param addon_manifests: Manifest for addons, see http://ahal.ca/blog/2011/bulk-installing-fx-addons/
+        :param apps: Dictionary or class of webapps to install
+        :param preferences: Dictionary or class of preferences
+        :param locations: locations to proxy
+        :param proxy: setup a proxy - dict of server-loc,server-port,ssl-port
+        :param restore: If true remove all installed addons preferences when cleaning up
+        """
 
         # if true, remove installed addons/prefs afterwards
         self.restore = restore
@@ -36,7 +47,8 @@ class Profile(object):
         self.written_prefs = set()
 
         # our magic markers
-        self.delimeters = ('#MozRunner Prefs Start', '#MozRunner Prefs End')
+        nonce = '%s %s' % (str(time.time()), uuid.uuid4())
+        self.delimeters = ('#MozRunner Prefs Start %s' % nonce,'#MozRunner Prefs End %s' % nonce)
 
         # Handle profile creation
         self.create_new = not profile
@@ -77,6 +89,10 @@ class Profile(object):
         self.addon_manager = AddonManager(self.profile)
         self.addon_manager.install_addons(addons, addon_manifests)
 
+        # handle webapps
+        self.webapps = WebappCollection(profile=self.profile, apps=apps)
+        self.webapps.update_manifests()
+
     def exists(self):
         """returns whether the profile exists or not"""
         return os.path.exists(self.profile)
@@ -97,10 +113,33 @@ class Profile(object):
                       locations=self._locations,
                       proxy = self._proxy)
 
+    @classmethod
+    def clone(cls, path_from, path_to=None, **kwargs):
+        """Instantiate a temporary profile via cloning
+        - path: path of the basis to clone
+        - kwargs: arguments to the profile constructor
+        """
+        if not path_to:
+            tempdir = tempfile.mkdtemp() # need an unused temp dir name
+            rmtree(tempdir) # copytree requires that dest does not exist
+            path_to = tempdir
+        copytree(path_from, path_to)
+
+        def cleanup_clone(fn):
+            """Deletes a cloned profile when restore is True"""
+            def wrapped(self):
+                fn(self)
+                if self.restore and os.path.exists(self.profile):
+                        rmtree(self.profile, onerror=self._cleanup_error)
+            return wrapped
+
+        c = cls(path_to, **kwargs)
+        c.__del__ = c.cleanup = types.MethodType(cleanup_clone(cls.cleanup), c)
+        return c
+
     def create_new_profile(self):
-        """Create a new clean profile in tmp which is a simple empty folder"""
-        profile = tempfile.mkdtemp(suffix='.mozrunner')
-        return profile
+        """Create a new clean temporary profile which is a simple empty folder"""
+        return tempfile.mkdtemp(suffix='.mozrunner')
 
 
     ### methods for preferences
@@ -108,7 +147,6 @@ class Profile(object):
     def set_preferences(self, preferences, filename='user.js'):
         """Adds preferences dict to profile preferences"""
 
-
         # append to the file
         prefs_file = os.path.join(self.profile, filename)
         f = open(prefs_file, 'a')
@@ -118,18 +156,15 @@ class Profile(object):
             # note what files we've touched
             self.written_prefs.add(filename)
 
-
-            if isinstance(preferences, dict):
-                # order doesn't matter
-                preferences = preferences.items()
+            # opening delimeter
+            f.write('\n%s\n' % self.delimeters[0])
 
             # write the preferences
-            f.write('\n%s\n' % self.delimeters[0])
-            _prefs = [(simplejson.dumps(k), simplejson.dumps(v) )
-                      for k, v in preferences]
-            for _pref in _prefs:
-                f.write('user_pref(%s, %s);\n' % _pref)
+            Preferences.write(f, preferences)
+
+            # closing delimeter
             f.write('%s\n' % self.delimeters[1])
+
         f.close()
 
     def pop_preferences(self, filename):
@@ -216,21 +251,25 @@ class Profile(object):
                 self.clean_preferences()
                 self.addon_manager.clean_addons()
                 self.permissions.clean_db()
+                self.webapps.clean()
 
     __del__ = cleanup
 
 class FirefoxProfile(Profile):
     """Specialized Profile subclass for Firefox"""
+
     preferences = {# Don't automatically update the application
                    'app.update.enabled' : False,
                    # Don't restore the last open set of tabs if the browser has crashed
                    'browser.sessionstore.resume_from_crash': False,
-                   # Don't check for the default web browser
+                   # Don't check for the default web browser during startup
                    'browser.shell.checkDefaultBrowser' : False,
                    # Don't warn on exit when multiple tabs are open
                    'browser.tabs.warnOnClose' : False,
                    # Don't warn when exiting the browser
                    'browser.warnOnQuit': False,
+                   # Don't send Firefox health reports to the production server
+                   'datareporting.healthreport.documentServerURI' : 'http://%(server)s/healthreport/',
                    # Only install add-ons from the profile and the application scope
                    # Also ensure that those are not getting disabled.
                    # see: https://developer.mozilla.org/en/Installing_extensions
@@ -244,13 +283,58 @@ class FirefoxProfile(Profile):
                    'extensions.update.enabled'    : False,
                    # Don't open a dialog to show available add-on updates
                    'extensions.update.notifyUser' : False,
-                   # Suppress automatic safe mode after crashes
-                   'toolkit.startup.max_resumed_crashes' : -1,
                    # Enable test mode to run multiple tests in parallel
                    'focusmanager.testmode' : True,
+                   # Suppress delay for main action in popup notifications
+                   'security.notification_enable_delay' : 0,
+                   # Suppress automatic safe mode after crashes
+                   'toolkit.startup.max_resumed_crashes' : -1,
+                   # Don't report telemetry information
+                   'toolkit.telemetry.enabled' : False,
+                   'toolkit.telemetry.enabledPreRelease' : False,
+                   }
+
+class FirefoxMetroProfile(Profile):
+    """Specialized Profile subclass for Firefox Metro"""
+
+    preferences = {# Don't automatically update the application for desktop and metro build
+                   'app.update.enabled' : False,
+                   'app.update.metro.enabled' : False,
+                   # Don't restore the last open set of tabs if the browser has crashed
+                   'browser.sessionstore.resume_from_crash': False,
+                   # Don't check for the default web browser during startup
+                   'browser.shell.checkDefaultBrowser' : False,
+                   # Don't send Firefox health reports to the production server
+                   'datareporting.healthreport.documentServerURI' : 'http://%(server)s/healthreport/',
+                   # Only install add-ons from the profile and the application scope
+                   # Also ensure that those are not getting disabled.
+                   # see: https://developer.mozilla.org/en/Installing_extensions
+                   'extensions.enabledScopes' : 5,
+                   'extensions.autoDisableScopes' : 10,
+                   # Don't install distribution add-ons from the app folder
+                   'extensions.installDistroAddons' : False,
+                   # Dont' run the add-on compatibility check during start-up
+                   'extensions.showMismatchUI' : False,
+                   # Disable strict compatibility checks to allow add-ons enabled by default
+                   'extensions.strictCompatibility' : False,
+                   # Don't automatically update add-ons
+                   'extensions.update.enabled'    : False,
+                   # Don't open a dialog to show available add-on updates
+                   'extensions.update.notifyUser' : False,
+                   # Enable test mode to run multiple tests in parallel
+                   'focusmanager.testmode' : True,
+                   # Suppress delay for main action in popup notifications
+                   'security.notification_enable_delay' : 0,
+                   # Suppress automatic safe mode after crashes
+                   'toolkit.startup.max_resumed_crashes' : -1,
+                   # Don't report telemetry information
+                   'toolkit.telemetry.enabled' : False,
+                   'toolkit.telemetry.enabledPreRelease' : False,
                    }
 
 class ThunderbirdProfile(Profile):
+    """Specialized Profile subclass for Thunderbird"""
+
     preferences = {'extensions.update.enabled'    : False,
                    'extensions.update.notifyUser' : False,
                    'browser.shell.checkDefaultBrowser' : False,
